@@ -39,7 +39,7 @@ class GnuFCompiler(FCompiler):
         m = re.match(r'GNU Fortran\s+95.*?([0-9-.]+)', version_string)
         if m:
             return ('gfortran', m.group(1))
-        m = re.match(r'GNU Fortran.*?([0-9-.]+)', version_string)
+        m = re.match(r'GNU Fortran.*?\-?([0-9-.]+)', version_string)
         if m:
             v = m.group(1)
             if v.startswith('0') or v.startswith('2') or v.startswith('3'):
@@ -202,6 +202,19 @@ class GnuFCompiler(FCompiler):
         opt.append('-funroll-loops')
         return opt
 
+    def _c_arch_flags(self):
+        """ Return detected arch flags from CFLAGS """
+        from distutils import sysconfig
+        try:
+            cflags = sysconfig.get_config_vars()['CFLAGS']
+        except KeyError:
+            return []
+        arch_re = re.compile(r"-arch\s+(\w+)")
+        arch_flags = []
+        for arch in arch_re.findall(cflags):
+            arch_flags += ['-arch', arch]
+        return arch_flags
+
     def get_flags_arch(self):
         return []
 
@@ -214,7 +227,17 @@ class Gnu95FCompiler(GnuFCompiler):
         v = self.gnu_version_match(version_string)
         if not v or v[0] != 'gfortran':
             return None
-        return v[1]
+        v = v[1]
+        if v>='4.':
+            # gcc-4 series releases do not support -mno-cygwin option
+            pass
+        else:
+            # use -mno-cygwin flag for gfortran when Python is not Cygwin-Python
+            if sys.platform == 'win32':
+                for key in ['version_cmd', 'compiler_f77', 'compiler_f90',
+                            'compiler_fix', 'linker_so', 'linker_exe']:
+                    self.executables[key].append('-mno-cygwin')
+        return v
 
     # 'gfortran --version' results:
     # XXX is the below right?
@@ -238,12 +261,6 @@ class Gnu95FCompiler(GnuFCompiler):
         'linker_exe'   : [None, "-Wall"]
         }
 
-    # use -mno-cygwin flag for g77 when Python is not Cygwin-Python
-    if sys.platform == 'win32':
-        for key in ['version_cmd', 'compiler_f77', 'compiler_f90',
-                    'compiler_fix', 'linker_so', 'linker_exe']:
-            executables[key].append('-mno-cygwin')
-
     module_dir_switch = '-J'
     module_include_switch = '-I'
 
@@ -254,8 +271,14 @@ class Gnu95FCompiler(GnuFCompiler):
         if not sys.platform == 'darwin':
             return []
         arch_flags = []
+        # get arches the C compiler gets.
+        c_archs = self._c_arch_flags()
+        if "i386" in c_archs:
+            c_archs[c_archs.index("i386")] = "i686"
+        # check the arches the Fortran compiler supports, and compare with
+        # arch flags from C compiler
         for arch in ["ppc", "i686", "x86_64", "ppc64"]:
-            if _can_target(cmd, arch):
+            if _can_target(cmd, arch) and arch in c_archs:
                 arch_flags.extend(["-arch", arch])
         return arch_flags
 
